/***********************************************************************
***********************************************************************/

#include "Examples/interface/Analysis_Ex.h"
#include <iostream>
#include <sstream>
#include <fstream>
#include <iomanip>
#include "TSimpleArray.h"
#include <vector>
#include "TLorentzVector.h"
#include "TF2.h"
#include "SmearUtil.h"
#include "TRandom3.h"

using namespace std;

TRandom3 grandom(0);  // a new seed is set everytime Delphes is run
string BTAG_func_b      = "0.4  + 0*x + 0*y";        // efficiency functions (x=Pt, y=eta)  in ROOT::TF2 format
string BTAG_func_c      = "0.1  + 0*x + 0*y";
string BTAG_func_l      = "0.01 + 0*x + 0*y";
string BTAG_func_g      = "0.01 + 0*x + 0*y";
double CEN_max_tracker   = 2.5;
const float ptmax = 9E6; // GeV/c
TF2 efficiencyB("efficiencyB",BTAG_func_b.c_str(),0,ptmax,-CEN_max_tracker,CEN_max_tracker); // efficiency function wrt Pt
TF2 efficiencyC("efficiencyC",BTAG_func_c.c_str(),0,ptmax,-CEN_max_tracker,CEN_max_tracker); // efficiency function wrt Pt
TF2 efficiencyL("efficiencyL",BTAG_func_l.c_str(),0,ptmax,-CEN_max_tracker,CEN_max_tracker); // efficiency function wrt Pt



/*
float DeltaRR(const float phi1, const float eta1, const float phi2, const float eta2) {
  return sqrt(pow(DeltaPhi(phi1,phi2),2) + pow(eta1-eta2,2));
}


int Bjets(const TSimpleArray<TRootC::GenParticle> &subarray, const float& eta, const float& phi) {
  int flav_max=0;
  int Ppid=0;
  RESOLution* DET = new RESOLution();
  double JET_coneradius   = DET->JET_coneradius;

  if(subarray.GetEntries()>0) {
      for(int i=0; i < subarray.GetEntries();i++) { // should have pt>PT_JETMIN and a small cone radius (r<CONE_JET)
      float genDeltaR = DeltaRR(subarray[i]->Phi,subarray[i]->Eta,phi,eta);
          if(genDeltaR < JET_coneradius && (abs( subarray[i]->PID) < 6 && abs(subarray[i]->PID) > flav_max)) {
              flav_max=abs(subarray[i]->PID);
              Ppid=abs(subarray[i]->PID);
          }
      }
  }
  return Ppid;
}


bool Btaggedjet(const TLorentzVector &JET, const TSimpleArray<TRootC::GenParticle> &subarray) {

  string BTAG_func_b      = "0.4  + 0*x + 0*y";        // efficiency functions (x=Pt, y=eta)  in ROOT::TF2 format
  string BTAG_func_c      = "0.1  + 0*x + 0*y";
  string BTAG_func_l      = "0.01 + 0*x + 0*y";
  string BTAG_func_g      = "0.01 + 0*x + 0*y";
  double CEN_max_tracker   = 2.5;

  const float ptmax = 9E6; // GeV/c
  bool tag = false;
  string efficiency_formula="";
 // selects the correct formula
   int jet_id = Bjets(subarray,JET.Eta(),JET.Phi()); // gets the flavor of the jet
      switch (jet_id) {
          case 5: efficiency_formula = BTAG_func_b; break;
          case 4: efficiency_formula = BTAG_func_c; break;
          default: efficiency_formula = BTAG_func_l; break;
                  }
 
     TF2 efficiency("efficiency",efficiency_formula.c_str(),0,ptmax,-CEN_max_tracker,CEN_max_tracker); // efficiency function wrt Pt
     float x = grandom.Uniform(); // randomisation
     tag = (x < efficiency.Eval(JET.Pt(),JET.Eta())) ? true : false;
     return tag;
      }

*/


//******************************Debut de l'analyse****************************************
//*****************************************************************************************

Analysis_Ex::Analysis_Ex(string CardWithCuts,string LogName)
{
  string temp_string;
  istringstream curstring;
  
  ifstream fichier_a_lire(CardWithCuts.c_str());
  if(!fichier_a_lire.good()) {
    cout << "DataCardname " << CardWithCuts << " not found, use default values" << endl;
    return;
  }
  
  while (getline(fichier_a_lire,temp_string)) {
    curstring.clear(); // needed when using several times istringstream::str(string)
    curstring.str(temp_string);
    string varname;
    float value;
    
    if(strstr(temp_string.c_str(),"#")) { }//remove comments
    else if(strstr(temp_string.c_str(),"PT_ELEC")){curstring >> varname >> value; PT_ELEC = value;}
    else if(strstr(temp_string.c_str(),"PT_MUON")){curstring >> varname >> value; PT_MUON = value;}
    else if(strstr(temp_string.c_str(),"INV_MASS_LL")){curstring >> varname >> value; INV_MASS_LL = value;}
  }
  
  ofstream f_out(LogName.c_str(),ofstream::app);
  
  f_out<<"*******************************************************************"<<endl;
  f_out << left << setw(30) <<"Cut values used in the analysis: "<<""
        << right << setw(37) <<"-------------------------------------"<<"\n";
  f_out << left <<setw(50) << "Invariant mass of the leptons: "<<""
        << right <<setw(17) << INV_MASS_LL <<"\n";
  f_out<<"*******************************************************************"<<endl;


  
}

Analysis_Ex::~Analysis_Ex()
{
}

void Analysis_Ex::Run(ExRootTreeReader *treeReaderGen, ExRootTreeReader *treeReaderRec, ExRootTreeReader *treeReaderTrig, ExRootTreeWriter *treeWriter)
{ 

 

    string name="main.ini";
    ifstream myfile;
    myfile.open(name.c_str(), ios::in);
    if (!myfile) {
      cerr << "\nGlobal::getIni(): Can't open input file:  " << name << endl;
      exit(1);
    } else {
        cout << "\nRead file=" << name << endl;
    }
    type=1;
    MaxEvents=10000000;
    string message;
    int    number;
    myfile >> message >> MaxEvents;
    myfile >> message >> number;
    myfile >> message >> type;
    xdebug=false;
    if (number==1) xdebug=true;
    myfile.close();








  minPTall=800;
  maxEta=0.8;
  cout << "-- Required number of events=" << MaxEvents << endl;
  cout << "-- Required debug mode =" << xdebug << endl;
  cout << "-- Required type mode =" << type << endl;
  cout << "cut: minPTall=" << minPTall << endl;
  cout << "cut: maxEta=" << maxEta << endl;
 
   // set histograms
  // initialisations 
  debug = new TH1D("debug", "debug", 30, 0, 30);

  btag  = new TH1D("btag", "Btagging", 5, 0, 5.0);

  jet_pt  = new TH1D("jet_pt", "Jet(pt)", 100, 500, 2000.0);
  jet_eta = new TH1D("jet_eta", "Jet(eta)", 40, -2.0, 2.0);
  jet_mass  = new TH1D("jet_mass", "Jet(mass)", 80, 0, 400);
  jet_hem  = new TH1D("jet_hem", "EHoverEE", 40, 0, 10.0);

  jet_pt_b  = new TH1D("jet_pt_b", "Jet(pt) Btag", 100, 500, 2000.0);
  jet_eta_b = new TH1D("jet_eta_b", "Jet(eta) Btag", 40, -2.0, 2.0);
  jet_mass_b  = new TH1D("jet_mass_b", "Jet(mass) Btag", 80, 0, 400);
 
  top_pt  = new TH1D("top_pt", "Top(pt)", 100, 500, 2000.0);
  top_eta = new TH1D("top_eta", "Top(eta)", 40, -2.0, 2.0);






  total=0;//initialisation of total number of events
  cut_trig=0;
  cut_1=0;//initialisation of counter for cut 1
  cut_2=0;
  //access the branches************************
  //to get the generator level information
  const TClonesArray *GEN = treeReaderGen->UseBranch("Particle");
  
  //to get the reconstructed level information
  const TClonesArray *JET    = treeReaderRec->UseBranch("Jet");
  const TClonesArray *TAUJET = treeReaderRec->UseBranch("TauJet");
  const TClonesArray *PHOTO  = treeReaderRec->UseBranch("Photon");
  const TClonesArray *ELEC   = treeReaderRec->UseBranch("Electron");
  const TClonesArray *MUON   = treeReaderRec->UseBranch("Muon");
  const TClonesArray *TRACKS = treeReaderRec->UseBranch("Tracks");
  const TClonesArray *CALO   = treeReaderRec->UseBranch("CaloTower");
  
  //to get the VFD reconstructed level information
  const TClonesArray *ZDC    = treeReaderRec->UseBranch("ZDChits");
  const TClonesArray *RP220 = treeReaderRec->UseBranch("RP220hits");
  const TClonesArray *FP420 = treeReaderRec->UseBranch("FP420hits");
  
  //to get the trigger information
  const TClonesArray *TRIGGER  = treeReaderTrig->UseBranch("TrigResult");

  //Define the branches that will be filled during the analysis
  //ExRootTreeBranch *INVMASS = treeWriter->NewBranch("INVMass", TRootInvm::Class());
  //TRootInvm *inv_mass; 
  //*******************************************
  
  //run on the events
  Long64_t entry, allEntries = treeReaderRec->GetEntries();
  cout << "** Chain contains " << allEntries << " events" << endl;
  total=allEntries;

  //general information
  float E,Px,Py,Pz;
  float PT,Eta,Phi;

  //lepton information
  bool IsolFlag;
 
  //bjet information
  bool Btag;

  //Particle level information
  int PID, Status, M1,M2,D1,D2;
  float Charge, T, X, Y, Z, M;


  //VFD information
  float S,q2,Tx,Ty;  
  int side;
  bool hadronic_hit; 
  int nev=0;
  int ntop=0;
        
  for(entry = 0; entry < allEntries; ++entry)
    {
      treeReaderGen->ReadEntry(entry);//access information of generated information
      treeReaderRec->ReadEntry(entry);//access information of reconstructed information
      treeReaderTrig->ReadEntry(entry);//access information of Trigger information
 

       if (nev<=10 &&
       nev<=100 && (nev%10) == 0 ||
       nev<=10000 && (nev%1000) == 0  ||
       nev>=10000 && (nev%10000) == 0 ) {
          cout << "Events= " << nev << endl; }
       // counter
       nev++;

       // stop after max event
       if (nev>MaxEvents) break;



      vector<TLorentzVector> bquarks;
      vector<TLorentzVector> cquarks;
     
      //*****************************************************
      //Example how to run on the generator level information
      //*****************************************************


   int ev_tot=0;


   TIter itGen((TCollection*)GEN);
   TRootC::GenParticle *gen;
   TRootC::GenParticle *part;

   // creates the array of particles
   TSimpleArray<TRootC::GenParticle>  array;
   itGen.Reset();
   while( (part = (TRootC::GenParticle*) itGen.Next()) )
     {
       array.Add(part);
     }


      itGen.Reset();
      while( (gen = (TRootC::GenParticle*) itGen.Next()) )
	{
          PID      = gen->PID;      // particle HEP ID number 
          Status   = gen->Status;   // particle status 
          M1       = gen->M1;       // particle 1st mother 
          M2       = gen->M2;       // particle 2nd mother 
          D1       = gen->D1;       // particle 1st daughter 
          D2       = gen->D2;       // particle 2nd daughter 
          Charge = gen->Charge;   // electrical charge
	  float PT      = gen->PT;
          float Eta     = gen->Eta; 
          float Phi     = gen->Phi;

          T      = gen->T;        // particle vertex position (t component)
          X      = gen->X;        // particle vertex position (x component) 
          Y      = gen->Y;        // particle vertex position (y component) 
          Z      = gen->Z;        // particle vertex position (z component)
          M      = gen->M;        // particle mass

          if (PT>100 && abs(PID)==5 && abs(Eta)<2){ // b-jet  
             TLorentzVector l;
             l.SetPtEtaPhiM(PT,Eta,Phi,M);
             bquarks.push_back(l);
          } 

          if (PT>100 && abs(PID)==4 && abs(Eta)<2.){ // charm  
             TLorentzVector l;
             l.SetPtEtaPhiM(PT,Eta,Phi,M);
             cquarks.push_back(l);
          }


          // take only 1 or 2. Do  not take dublicates
          // if (abs(PID)==5 || abs(PID)==6 )  cout << PID << " " << PT << " " << Eta << endl;
          if (PT>650 && abs(PID)==6 && abs(Eta)<0.8) { 
                //     cout << " moter=" << array[M1]->PID << endl; 
                if (abs(array[M1]->PID) != 6) {  
                top_pt->Fill(PT); top_eta->Fill(Eta); ntop++; ev_tot++; 
                // cout << Eta << " " <<  Status <<  " " << M1 << endl;}
                 }


	}
       } // end loop over truth  


      // consider only top
      if (type>10) {
         // take only tops 
         cout << "This event has Nr of tops=" << ev_tot <<  " total found so far=" << ntop << endl;
         if (ev_tot == 0) continue;
        } // end top

 
      /*      
      //Example how to run on the reconstructed objects
      //access the Electron branch
      TIter itElec((TCollection*)ELEC);
      TRootElectron *elec;
      itElec.Reset();
      while( (elec = (TRootElectron*) itElec.Next()) )
	{
	  E        = elec->E;        // particle energy in GeV
	  Px       = elec->Px;       // particle momentum vector (x component) in GeV
	  Py       = elec->Py;       // particle momentum vector (y component) in GeV
	  Pz       = elec->Pz;       // particle momentum vector (z component) in GeV
	  
	  PT       = elec->PT;       // particle transverse momentum in GeV
	  Eta      = elec->Eta;      // particle pseudorapidity
	  Phi      = elec->Phi;      // particle azimuthal angle in rad
	  IsolFlag = elec->IsolFlag;  // is the particule isolated?
	}
      //Running on the muon branch is identical:
      TIter itMuon((TCollection*)MUON);
      TRootMuon *muon;
      itMuon.Reset();
      while( (muon = (TRootMuon*) itMuon.Next()) ){}
      
      //access the Photon branch
      TIter itGam((TCollection*)PHOTO);
      TRootPhoton *gam;
      itGam.Reset();
      while( (gam = (TRootPhoton*) itGam.Next()) )
	{
	  E        = gam->E;        // particle energy in GeV
	  Px       = gam->Px;       // particle momentum vector (x component) in GeV
	  Py       = gam->Py;       // particle momentum vector (y component) in GeV
	  Pz       = gam->Pz;       // particle momentum vector (z component) in GeV
	  
          PT       = gam->PT;       // particle transverse momentum in GeV
	  Eta      = gam->Eta;      // particle pseudorapidity
	  Phi      = gam->Phi;      // particle azimuthal angle in rad
	}

       */
      
      //access the jet branch
      TIter itJet((TCollection*)JET);
      TRootJet *jet;
      itJet.Reset();
      while( (jet = (TRootJet*) itJet.Next()) )
	{
	  E        = jet->E;        // particle energy in GeV
	  Px       = jet->Px;       // particle momentum vector (x component) in GeV
	  Py       = jet->Py;       // particle momentum vector (y component) in GeV
	  Pz       = jet->Pz;       // particle momentum vector (z component) in GeV
	  
	  PT       = jet->PT;       // particle transverse momentum in GeV
	  Eta      = jet->Eta;      // particle pseudorapidity
	  Phi      = jet->Phi;      // particle azimuthal angle in rad
	  Btag     = jet->Btag;     // is the jet BTagged 
          float hem= jet->EHoverEE; 
          int ntracks=jet->NTracks;
          int ncalo=jet->NCalo;

        // fix b-tag
         Btag=false;
        // for all jets- 1% mistag
        float xr = grandom.Uniform(); // randomisation
        if (xr<efficiencyL.Eval(PT,Eta)) Btag=true;

          // b-jet
          for(int i = 0; i<bquarks.size(); i++){
          TLorentzVector bb = (TLorentzVector)bquarks.at(i);
          double phiT = bb.Phi();
          double ptT =  bb.Perp();
          double etaT = bb.PseudoRapidity();
          double Deta=fabs(Eta-etaT);
          double Dphi=fabs(Phi-phiT);
          double R=sqrt(Deta*Deta+Dphi*Dphi);
          if (R<0.6)  {
           //Btag = (x < efficiency.Eval(PT,Eta)) ? true : false;
           xr = grandom.Uniform(); // randomisation
           if (xr < efficiencyB.Eval(PT,Eta)) Btag=true;
          }
         }
          // c-quark mistag
          for(int i = 0; i<cquarks.size(); i++){
          TLorentzVector bb = (TLorentzVector)cquarks.at(i);
          double phiT = bb.Phi();
          double ptT =  bb.Perp();
          double etaT = bb.PseudoRapidity();
          double Deta=fabs(Eta-etaT);
          double Dphi=fabs(Phi-phiT);
          double R=sqrt(Deta*Deta+Dphi*Dphi);
          if (R<0.6)  {
           //Btag = (x < efficiency.Eval(PT,Eta)) ? true : false;
           xr = grandom.Uniform(); // randomisation
           if (xr < efficiencyC.Eval(PT,Eta)) Btag=true;
          }
         }
   



            TLorentzVector l;
            l.SetPxPyPzE(Px,Py,Pz,E);
            double phi = l.Phi();
            //double pt =  l.Perp();
            double m=l.M();
            if (PT<minPTall)      continue;
            //double eta = l.PseudoRapidity();
            if (fabs(Eta)>maxEta) continue;
            jet_pt->Fill(PT);
            jet_eta->Fill(Eta);
            jet_mass->Fill(m);
            jet_hem->Fill(hem);



            float x=0;
            if (Btag) x=1;
            btag->Fill(x);

            if (Btag) {
              jet_pt_b->Fill(PT);
              jet_eta_b->Fill(Eta);
              jet_mass_b->Fill(m);

            } 
	}

/*
      //Running on the tau-jet branch is identical:
      TIter itTaujet((TCollection*)TAUJET);
      TRootTauJet *taujet;
      itTaujet.Reset();
      while( (taujet = (TRootTauJet*) itTaujet.Next()) ){}
      
      //access the track branch
      TIter itTrack((TCollection*)TRACKS);
      TRootTracks *tracks;
      itTrack.Reset();
      while( (tracks = (TRootTracks*) itTrack.Next()) )
	{
	  E        = tracks->E;        // particle energy in GeV
	  Px       = tracks->Px;       // particle momentum vector (x component) in GeV
	  Py       = tracks->Py;       // particle momentum vector (y component) in GeV
	  Pz       = tracks->Pz;       // particle momentum vector (z component) in GeV
	  
	  PT       = tracks->PT;       // particle transverse momentum in GeV
	  Eta      = tracks->Eta;      // particle pseudorapidity
	  Phi      = tracks->Phi;      // particle azimuthal angle in rad
	}
      
      //Running on the calo branch is identical:
      TIter itCalo((TCollection*)CALO);
      TRootCalo *calo;
      itCalo.Reset();
      while( (calo = (TRootCalo*) itCalo.Next()) ){}
      
      //Example how to run on the VFD reconstructed objects
      //access the ZDC branch
      TIter itZdc((TCollection*)ZDC);
      TRootZdcHits *zdc;
      itZdc.Reset();
      while( (zdc = (TRootZdcHits*) itZdc.Next()) )
	{
	  E   = zdc->E;     // particle energy in GeV
	  T   = zdc->T;     // time of flight [s]
	  Px  = zdc->Px;    // particle momentum vector (x component) in GeV
	  Py  = zdc->Py;    // particle momentum vector (y component) in GeV
	  Pz  = zdc->Pz;    // particle momentum vector (z component) in GeV
	  
	  PT  = zdc->PT;    // particle transverse momentum in GeV
	  Eta = zdc->Eta;   // particle pseudorapidity
	  Phi = zdc->Phi;   // particle azimuthal angle in rad
	  side  = zdc->side;  // -1 or +1
          //hadronic_hit = zdc->hadronic_hit; // true if neutron, false if photon
	}
      
      //access the RP220 branch
      TIter itRp220((TCollection*)RP220);
      TRootRomanPotHits *rp220;
      itRp220.Reset();
      
      while( (rp220 = (TRootRomanPotHits*) itRp220.Next()) )
	{
	  //T   = rp220->T;     // time of flight to the detector [s]
	  S   = rp220->S;     // distance to the IP [m]
	  E   = rp220->E;     // reconstructed energy [GeV]
	  q2  = rp220->q2;    // reconstructed squared momentum transfer [GeV^2]
	  
	  X   = rp220->X;     // horizontal distance to the beam [um]
	  Y   = rp220->Y;     // vertical distance to the beam [um]
	  
	  Tx  = rp220->Tx;    // angle of the momentum in the horizontal (x,z) plane [urad]
	  Ty  = rp220->Ty;    // angle of the momentum in the verical (y,z) plane [urad]
	 
          T   = rp220->T;     // time of arrival of the particle in the detector [s] 
	  side  = rp220->side;  // -1 or 1
	}
      //running on FP420 branch is identical
      TIter  itFp420((TCollection*)FP420);
      TRootRomanPotHits *fp420;
      itFp420.Reset();
      while( (fp420 = (TRootRomanPotHits*) itFp420.Next()) ){}
      
      //Example how to run on the trigger information
      
      TRootTrigger *trig;
      int NumTrigBit = TRIGGER->GetEntries();
      //get the global response of the trigger
      
      bool GlobalResponse=false;
      if(NumTrigBit!=0)GlobalResponse=true;
      //cout<<"GlobalResponse "<<GlobalResponse<<endl;
      for(int i=0; i < NumTrigBit-1; i++){
	trig = (TRootTrigger*)TRIGGER->At(i);
	//cout<<"The event has been accepted by the trigger number: "<<trig->Accepted<<endl;
      }
      
     //Example of a very small analysis

     TLorentzVector Lept[2];

     if(NumTrigBit==0)continue;    //event not accepted by the trigger
     cut_trig++;//event accepted
        
     TSimpleArray<TRootElectron> el=SubArrayEl(ELEC,PT_ELEC);//the central isolated electrons, pt > PT_ELEC GeV 
     TSimpleArray<TRootMuon> mu=SubArrayMu(MUON,PT_MUON);//the central isolated electrons, pt > PT_MUON GeV 

     Int_t numElec=el.GetEntries();

     if(el.GetEntries()+mu.GetEntries()!=2)continue;//Exactly 2 isolated leptons are needed
     cut_1++;//event accepted
     for(Int_t i=0;i < numElec; i++)Lept[i].SetPxPyPzE(el[i]->Px,el[i]->Py,el[i]->Pz,el[i]->E);
     for(Int_t k = numElec; k < (numElec+mu.GetEntries()); k++)Lept[k].SetPxPyPzE(mu[k-numElec]->Px,mu[k-numElec]->Py,mu[k-numElec]->Pz,mu[k-numElec]->E);
    cout<<"normalement il y a quelque chose... "<<endl;
     //Example how to white a branch in the output file
     inv_mass=(TRootInvm*) INVMASS->NewEntry();
     inv_mass->M=(Lept[0]+Lept[1]).M();
 
     if((Lept[0]+Lept[1]).M() > INV_MASS_LL )continue;// the invariant mass should be < INV_MASS_LL
     cut_2++;//event accepted

     treeWriter->Fill(); 

*/
   }

// treeWriter->Write();


}

void Analysis_Ex::WriteOutput(string LogName)
{
  ofstream f_out(LogName.c_str(),ofstream::app);

  f_out<<"*******************************************************************"<<endl;
  f_out << left  << setw(20) << "Numer of Events "<<""
        << right << setw(15) << total <<"\n";  
  f_out << left  << setw(17) << " Accepted by the trigger "<<""
        << right << setw(20) << cut_trig <<"\n";
  f_out << left  << setw(17) <<" 2 leptons "<< ""
        << right << setw(20) << cut_1 << ""        
        << right << setw(15) << cut_1/total << "\n";        
  f_out << left  << setw(17) <<" Invariant mass "<< ""
        << right << setw(20) << cut_2 << ""
        << right << setw(15) << cut_2/total << "\n";
  f_out<<"*******************************************************************"<<endl;
  f_out<<"                                                                   "<<endl;

}

TSimpleArray<TRootElectron> Analysis_Ex::SubArrayEl(const TClonesArray *ELEC,float pt)
{
    TIter itElec((TCollection*)ELEC);
    TRootElectron *elec;
    itElec.Reset();
    TSimpleArray<TRootElectron> array;
      while( (elec = (TRootElectron*) itElec.Next()) )
        {
         if(elec->PT<pt)continue;
         array.Add(elec);
        }
    return array;
}

TSimpleArray<TRootMuon> Analysis_Ex::SubArrayMu(const TClonesArray *MUON,float pt)
{
   TIter itMuon((TCollection*)MUON);
   TRootMuon *muon;
   itMuon.Reset();
   TSimpleArray<TRootMuon> array;
   while( (muon = (TRootMuon*) itMuon.Next()) )
     {
        if(muon->PT<pt)continue;
        array.Add(muon);
     }
   return array;
}

